import { schemaToTemplateContext } from 'graphql-codegen-core';
import { helpers } from 'graphql-codegen-plugin-handlebars-helpers';
import { resolveExternalModuleAndFn } from 'graphql-codegen-plugin-helpers';
import * as enumTemplate from './enum.handlebars';
import * as type from './type.handlebars';
import * as rootTemplate from './root.handlebars';
import * as Handlebars from 'handlebars';
import { getOptionals, getType, getEnumValue, getScalarType, defineMaybe, importEnum, concat } from './helpers';
import { pascalCase } from 'change-case';
export * from './helpers';
export const DEFAULT_SCALARS = {
    String: 'string',
    Int: 'number',
    Float: 'number',
    Boolean: 'boolean',
    ID: 'string'
};
export function initCommonTemplate(hbs, schema, config) {
    const scalars = Object.assign({}, DEFAULT_SCALARS, (config.scalars || {}));
    let namingConventionMap;
    if (typeof config.namingConvention === 'undefined') {
        namingConventionMap = {
            default: pascalCase,
            enumValues: pascalCase,
            typeNames: pascalCase
        };
    }
    else if (typeof config.namingConvention === 'string') {
        namingConventionMap = {
            default: config.namingConvention,
            enumValues: config.namingConvention,
            typeNames: config.namingConvention
        };
    }
    else {
        namingConventionMap = {
            default: config.namingConvention.default || pascalCase,
            enumValues: config.namingConvention.enumValues || config.namingConvention.default || pascalCase,
            typeNames: config.namingConvention.typeNames || config.namingConvention.default || pascalCase
        };
    }
    const convert = (str, kind = 'default') => {
        // Workaround for use-case as `{{ convert something }}` without specifying. The "kind" arguemnt
        // will contains the "options" value of Handlebars, in this case we need to override it to be "default".
        if (typeof kind === 'object') {
            kind = 'default';
        }
        const baseConvertFn = !namingConventionMap[kind] || namingConventionMap[kind] === 'keep'
            ? (str) => str
            : typeof namingConventionMap[kind] === 'string'
                ? resolveExternalModuleAndFn(namingConventionMap[kind])
                : namingConventionMap[kind];
        if (str.charAt(0) === '_') {
            const after = str.replace(/^(_*)(.*)/, (_match, underscorePrefix, typeName) => `${underscorePrefix}${baseConvertFn(typeName || '')}`);
            return after;
        }
        return baseConvertFn(str);
    };
    hbs.registerPartial('enum', enumTemplate);
    hbs.registerPartial('type', type);
    hbs.registerHelper('concat', concat);
    hbs.registerHelper('defineMaybe', defineMaybe);
    hbs.registerHelper('blockComment', helpers.blockComment);
    hbs.registerHelper('blockCommentIf', helpers.blockCommentIf);
    hbs.registerHelper('toComment', helpers.toComment);
    hbs.registerHelper('convert', convert);
    hbs.registerHelper('getOptionals', getOptionals);
    hbs.registerHelper('getEnumValue', getEnumValue);
    hbs.registerHelper('importEnum', importEnum);
    hbs.registerHelper('convertedType', getType(convert));
    hbs.registerHelper('toLowerCase', helpers.toLowerCase);
    hbs.registerHelper('toUpperCase', helpers.toUpperCase);
    hbs.registerHelper('times', helpers.times);
    hbs.registerHelper('stringify', helpers.stringify);
    hbs.registerHelper('ifDirective', helpers.ifDirective);
    hbs.registerHelper('ifCond', helpers.ifCond);
    hbs.registerHelper('getScalarType', getScalarType);
    hbs.registerHelper('unlessDirective', helpers.unlessDirective);
    hbs.registerHelper('toPrimitive', type => scalars[type] || type || '');
    const templateContext = schemaToTemplateContext(schema);
    return {
        templateContext: Object.assign({}, templateContext, { config, primitives: scalars }),
        convert,
        scalars
    };
}
export const plugin = async (schema, documents, config) => {
    const { templateContext } = initCommonTemplate(Handlebars, schema, config);
    return Handlebars.compile(rootTemplate)(templateContext);
};
//# sourceMappingURL=index.js.map